
class DKOrderRunErrorException(Exception):
    pass

class DKOrderRunFailedTestException(Exception):
    pass

class DKOrderRunWarningTestException(Exception):
    pass

class DKLoginError(Exception):
    pass

class DKApiToolConfigError(Exception):
    pass


class DKOrderStartException(Exception):
    pass

class DKTimeoutException(Exception):
    pass


import json
import requests
import time
import logging
import datakitchen_api_tools.dk_api_helpers as dk_api_helpers
from datakitchen_api_tools.DKOrder import DKOrder
from datakitchen_api_tools.DKExceptions import *

logger = logging.getLogger("datakitchen_api_tools.DkConnection")

class DKConnection():
    def __init__(self, hostname, username, password, timeout=1):
        self.hostname = hostname
        payload = {'username': username, 'password': password}
        r = requests.post('https://%s/v2/login' % hostname, data=payload)
        if r.text == 'Credentials are invalid':
            raise Exception
        self.token = r.text
        self.headers =  {'content-type': 'application/json', 'authorization': 'bearer ' + self.token}

    def createOrder(self, kitchen, recipe, variation, overrides={}):
        headers = {'content-type': 'application/json', 'authorization': 'bearer ' + self.token}
        url = 'https://%s/v2/order/create/%s/%s/%s' % (self.hostname, kitchen, recipe, variation)
        payload = {'parameters': overrides, "schedule" : "now"}
        return requests.put(url, headers=headers, data=json.dumps(payload))

    def safeCreateOrder(self, kitchen, recipe, variation, overrides={}, wait_time=10, timeout=50000, test_level=""):
        response = self.createOrder(kitchen, recipe, variation, overrides=overrides)
        if response.status_code != 200:
            logger.error("Error Starting Order %s %s %s." % (kitchen, recipe, variation))
            raise DKOrderException
        order = json.loads(response.text)
        order_id = order['order_id']
        logger.info(order_id)
        order_run = self.getOrder(kitchen, order_id)
        time.sleep(wait_time)
        time_running = 0
        while order_run.isOrderRunning() and time_running < timeout:
            time.sleep(wait_time)
            order_run = self.getOrder(kitchen, order_id)
            time_running += wait_time
        if time_running >= timeout:
            raise DKTimeoutException
        if not order_run.didOrderCompleteSuccesfully():
            logger.error("Order %s, %s, %s Failed." % (kitchen, recipe, variation))
            raise DKOrderRunErrorException
        if len(order_run.getFailedTests(test_level)) > 0:
            logger.error("Tests in Order  %s, %s, %s Failed." % (kitchen, recipe, variation))
            raise DKOrderRunFailedTestException
        return order_run

    def OrderInfo(self, kitchen, order_id):
        url = "https://%s/v2/order/details/%s" % ( self.hostname, kitchen)
        headers = {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + self.token}
        payload = {'order_id': order_id, 'summary': True, 'logs': True, 'timingresults': True,
                   'testresults': True, 'servingjson': True, "schedule": "now"}
        r = requests.post(url, headers=headers, data=json.dumps(payload))
        if "does not exist." in r.text:
            raise Exception("Order run id %s not found." % order_run_id )
        return DKOrder(json.loads(r.text))

    def OrderRunInfo(self, kitchen, order_run_id):
        url = "https://%s/v2/order/details/%s" % (self.hostname, kitchen)
        headers = {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + self.token}
        payload = {'serving_hid': order_run_id, 'testresults': True}
        r = requests.post(url, headers=headers, data=json.dumps(payload))
        if "does not exist." in r.text:
            raise Exception("Order run id %s not found." % order_run_id)
        return DKOrder(json.loads(r.text))

    def getOrder(self, kitchen, order_id):
        return self.OrderInfo(kitchen, order_id)

    def TestsFromOrderRun(self, kitchen, order_run_id):
        return self.getOrderRun(kitchen, order_run_id).getTests()

    def OrderStatus(self, kitchen, order_run):
        return self.getOrder(kitchen, order_run_id).getOrderStatus()



import re

def parseTestString(testStr):
    tests = list()
    lines = testStr.strip().split('\n')
    state_level = "Failed"
    state_step = "" #Others are known 
    for line in lines:
        line_type = getTypeOfTestLine(line)
        if line_type:
            if line_type[0] == "Level":
                state_level = line_type[1]
            elif line_type[0] == "Step":
                state_step = line_type[1]
            elif line_type[0] == "Test":
                tests.append((line_type[1], state_step, state_level, line_type[2]))
    return tests

def getTypeOfTestLine(line):
    line = line.strip()
    match = re.match(r'Tests: (\w+)', line)
    if match:
        return ("Level", match.group(1))
    match = re.match(r'Step \((.+)\)', line)
    if match:
        return ("Step", match.group(1))
    match = re.match(r'\d+\. (.+) \((.+)\)', line)
    if match:
        return ("Test", match.group(1), match.group(2))
    return None



import logging
import re

# TODO: Exceptions and Logging

logger = logging.getLogger("datakitchen_api_tools.DkOrder")


class DKOrder():
    def __init__(self, order_run):
        self.order_run = order_run

    def testsFromOrderRun(self):
        try:
            testStr = self.order_run['servings'][0]['testresults']
        except KeyError:
            #print("Error: No tests in OrderRun Object")
            raise Exception
        return self._parseTestString(testStr)

    def mostRecentOrderRun(self):
        return self.order_run['servings'][0]

    def getCurrentOrderRunStatus(self):
        return self.mostRecentOrderRun()['status']

    def didOrderCompleteSuccesfully(self):
        order_status = self.order_run['orders'][0]['order-status']
        order_run_status = self.getCurrentOrderRunStatus()
        if order_status == "COMPLETED_ORDER" and order_run_status == "COMPLETED_SERVING":
            return True
        return False

    def getFailedTests(self, test_level):
        if test_level == "":
            return list()
        failed_tests = list()
        testResults = self.testsFromOrderRun()
        for level in  ["Failed", "Warning", "Log"]:
            failed_tests += [test for test in testResults if test[2] == level]
            if level == test_level:
                break
        return failed_tests

    def getFailedTestsByName(self, test_names):
        for test in self.testsFromOrderRun():
            if test[2] != "Passed" and test[0] in test_names:
                return False
        return True


    def getOrderStatus(self):
        return self.order_run['orders'][0]['order-status']

    def isOrderRunning(self):
        status = self.order_run['orders'][0]['order-status']
        if status in ["ACTIVE_ORDER",  "PLANNED_SERVING"]:
            return True
        return False

    def _parseTestString(self, testStr):
        tests = list()
        lines = testStr.strip().split('\n')
        state_level = "Failed"
        state_step = ""  # Others are known
        for line in lines:
            line_type = self._getTypeOfTestLine(line)
            if line_type:
                if line_type[0] == "Level":
                    state_level = line_type[1]
                elif line_type[0] == "Step":
                    state_step = line_type[1]
                elif line_type[0] == "Test":
                    tests.append((line_type[1], state_step, state_level, line_type[2]))
        return tests

    def _getTypeOfTestLine(self, line):
        line = line.strip()
        match = re.match(r'Tests: (\w+)', line)
        if match:
            return ("Level", match.group(1))
        match = re.match(r'Step \((.+)\)', line)
        if match:
            return ("Step", match.group(1))
        match = re.match(r'\d+\. (.+) \((.+)\)', line)
        if match:
            return ("Test", match.group(1), match.group(2))
        return None



from unittest import TestCase
from dk_api_helpers import *

class test_dk_api_helpers(TestCase):
    def test_getTypeOfTestLine(self):
        self.assertEquals(getTypeOfTestLine(''), None)
        self.assertEquals(getTypeOfTestLine('skjasdljkdskdsjlkdasjkl'), None)
        typ,val  = getTypeOfTestLine('Tests: Warning')
        self.assertEquals(typ, "Level")
        self.assertEquals(val, "Warning")
        typ,val  = getTypeOfTestLine('Step (put-dispense)')
        self.assertEquals(typ, "Step")
        self.assertEquals(val, "put-dispense")
        typ,val, desc  = getTypeOfTestLine('7. test-confirmationnum-mismatch-2010 (6624 num_dupes_confnum == 0)')
        self.assertEquals(typ, "Test")
        self.assertEquals(val, "test-confirmationnum-mismatch-2010")
        self.assertEquals(desc, "6624 num_dupes_confnum == 0")

    def test_parseTestString(self):
        with open('./tests/static/PlatformTestResults1.txt','r') as f:
            testData = f.read()
        expected = [(u'test_check_Veeva_Beghou_file_dates_invalid_date', u'check_Veeva_Beghou_file_dates', u'Passed', u'0 count_Veeva_Beghou_tables_invalid_date == 0'), (u'test_check_Veeva_Beghou_duplicate_file_types', u'check_Veeva_Beghou_file_dates', u'Passed', u'0 count_Veeva_Beghou_duplicate_file_types == 0'), (u'test_check_Veeva_Beghou_files', u'check_Veeva_Beghou_file_dates', u'Passed', u'12 count_Veeva_Beghou_files == 12'), (u'test_create_redshift_schema', u'create_schema', u'Passed', u'0 count_tables_in_schema == 0'), (u'test_copy_Veeva_HCPS', u'put_Veeva_Beghou', u'Passed',u'246721 count_Veeva_HCPS > 0'), (u'test_copy_Veeva_AccountGroupOverview', u'put_Veeva_Beghou', u'Passed', '929 count_Veeva_AccountGroupOverview > 0'), (u'test_copy_Veeva_Interaction', u'put_Veeva_Beghou', u'Passed', u'472529 count_Veeva_Interaction > 0'), (u'test_copy_Veeva_AddressChanges', u'put_Veeva_Beghou', u'Passed', '10870 count_Veeva_AddressChanges > 0'), (u'test_copy_Veeva_Pathways', u'put_Veeva_Beghou', u'Passed', '1428 count_Veeva_Pathways > 0')]
        self.assertListEqual(parseTestString(testData), expected)



from DKConnection import DKConnection

conn = DKConnection("cloud.datakitchen.io", "armand+hm@datakitchen.io", "%Gun8D9M")
results = conn.safeCreateOrder("hemonc_sftp_automation", 'hemonc_sftp_automation' ,'test-variation')



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="datakitchen_api_tools",
    version="0.2.6",
    author="Armand Halbert",
    author_email="armand@datakitchen.io",
    description="Python libray for working with DataKitchen's API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/",
    test_suite='nose.collector',
    tests_require=['nose'],
    packages=setuptools.find_packages(),
    install_requires=['requests'],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Operating System :: OS Independent",
    ],
)
